@@ -24,10 +24,12 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include "gdkpixmap.h"
 #include "gdkinternals.h"
 #include "gdkpixbuf.h"
 #include "gdkscreen.h"
+#include "gdkalias.h"
 
 static GdkGC *gdk_pixmap_create_gc      (GdkDrawable     *drawable,
                                          GdkGCValues     *values,
@@ -88,12 +90,21 @@ static void   gdk_pixmap_draw_lines     (GdkDrawable     *drawable,
 					 GdkGC           *gc,
 					 GdkPoint        *points,
 					 gint             npoints);
-static void   gdk_pixmap_draw_glyphs    (GdkDrawable      *drawable,
-                                         GdkGC            *gc,
-                                         PangoFont        *font,
-                                         gint              x,
-                                         gint              y,
-                                         PangoGlyphString *glyphs);
+
+static void gdk_pixmap_draw_glyphs             (GdkDrawable      *drawable,
+						GdkGC            *gc,
+						PangoFont        *font,
+						gint              x,
+						gint              y,
+						PangoGlyphString *glyphs);
+static void gdk_pixmap_draw_glyphs_transformed (GdkDrawable      *drawable,
+						GdkGC            *gc,
+						PangoMatrix      *matrix,
+						PangoFont        *font,
+						gint              x,
+						gint              y,
+						PangoGlyphString *glyphs);
+
 static void   gdk_pixmap_draw_image     (GdkDrawable     *drawable,
                                          GdkGC           *gc,
                                          GdkImage        *image,
@@ -115,7 +126,10 @@ static void   gdk_pixmap_draw_pixbuf    (GdkDrawable     *drawable,
 					 GdkRgbDither     dither,
 					 gint             x_dither,
 					 gint             y_dither);
-
+static void  gdk_pixmap_draw_trapezoids (GdkDrawable     *drawable,
+					 GdkGC	         *gc,
+					 GdkTrapezoid    *trapezoids,
+					 gint             n_trapezoids);
 
 static void   gdk_pixmap_real_get_size  (GdkDrawable     *drawable,
                                          gint            *width,
@@ -130,6 +144,8 @@ static GdkImage* gdk_pixmap_copy_to_image (GdkDrawable *drawable,
 					   gint         width,
 					   gint         height);
 
+static cairo_surface_t *gdk_pixmap_ref_cairo_surface (GdkDrawable *drawable);
+
 static GdkVisual*   gdk_pixmap_real_get_visual   (GdkDrawable *drawable);
 static gint         gdk_pixmap_real_get_depth    (GdkDrawable *drawable);
 static void         gdk_pixmap_real_set_colormap (GdkDrawable *drawable,
@@ -199,8 +215,10 @@ gdk_pixmap_class_init (GdkPixmapObjectClass *klass)
   drawable_class->draw_segments = gdk_pixmap_draw_segments;
   drawable_class->draw_lines = gdk_pixmap_draw_lines;
   drawable_class->draw_glyphs = gdk_pixmap_draw_glyphs;
+  drawable_class->draw_glyphs_transformed = gdk_pixmap_draw_glyphs_transformed;
   drawable_class->draw_image = gdk_pixmap_draw_image;
   drawable_class->draw_pixbuf = gdk_pixmap_draw_pixbuf;
+  drawable_class->draw_trapezoids = gdk_pixmap_draw_trapezoids;
   drawable_class->get_depth = gdk_pixmap_real_get_depth;
   drawable_class->get_screen = gdk_pixmap_real_get_screen;
   drawable_class->get_size = gdk_pixmap_real_get_size;
@@ -208,6 +226,7 @@ gdk_pixmap_class_init (GdkPixmapObjectClass *klass)
   drawable_class->get_colormap = gdk_pixmap_real_get_colormap;
   drawable_class->get_visual = gdk_pixmap_real_get_visual;
   drawable_class->_copy_to_image = gdk_pixmap_copy_to_image;
+  drawable_class->ref_cairo_surface = gdk_pixmap_ref_cairo_surface;
 }
 
 static void
@@ -369,6 +388,20 @@ gdk_pixmap_draw_glyphs (GdkDrawable      *drawable,
   gdk_draw_glyphs (private->impl, gc, font, x, y, glyphs);
 }
 
+static void
+gdk_pixmap_draw_glyphs_transformed (GdkDrawable      *drawable,
+				    GdkGC            *gc,
+				    PangoMatrix      *matrix,
+				    PangoFont        *font,
+				    gint              x,
+				    gint              y,
+				    PangoGlyphString *glyphs)
+{
+  GdkPixmapObject *private = (GdkPixmapObject *)drawable;
+
+  gdk_draw_glyphs_transformed (private->impl, gc, matrix, font, x, y, glyphs);
+}
+
 static void
 gdk_pixmap_draw_image (GdkDrawable     *drawable,
                        GdkGC           *gc,
@@ -407,6 +440,17 @@ gdk_pixmap_draw_pixbuf (GdkDrawable     *drawable,
 		   dither, x_dither, y_dither);
 }
 
+static void
+gdk_pixmap_draw_trapezoids (GdkDrawable     *drawable,
+			    GdkGC	    *gc,
+			    GdkTrapezoid    *trapezoids,
+			    gint             n_trapezoids)
+{
+  GdkPixmapObject *private = (GdkPixmapObject *)drawable;
+
+  gdk_draw_trapezoids (private->impl, gc, trapezoids, n_trapezoids);
+}
+
 static void
 gdk_pixmap_real_get_size (GdkDrawable *drawable,
                           gint *width,
@@ -476,6 +520,12 @@ gdk_pixmap_copy_to_image (GdkDrawable     *drawable,
 				     width, height);
 }
 
+static cairo_surface_t *
+gdk_pixmap_ref_cairo_surface (GdkDrawable *drawable)
+{
+  return _gdk_drawable_ref_cairo_surface (((GdkPixmapObject*)drawable)->impl);
+}
+
 static GdkBitmap *
 make_solid_mask (GdkScreen *screen, gint width, gint height)
 {
@@ -630,3 +680,6 @@ gdk_pixmap_real_get_screen (GdkDrawable *drawable)
 {
     return gdk_drawable_get_screen (GDK_PIXMAP_OBJECT (drawable)->impl);
 }
+
+#define __GDK_PIXMAP_C__
+#include "gdkaliasdef.c"
