@@ -72,6 +72,7 @@ struct _GtkIconThemePrivate
   guint pixbuf_supports_svg : 1;
   
   char *current_theme;
+  char *fallback_theme;
   char **search_path;
   int search_path_len;
 
@@ -409,11 +410,15 @@ update_current_theme (GtkIconTheme *icon_theme)
   if (!priv->custom_theme)
     {
       gchar *theme = NULL;
+      gchar *fallback_theme = NULL;
+      gboolean changed = FALSE;
 
       if (priv->screen)
 	{
 	  GtkSettings *settings = gtk_settings_get_for_screen (priv->screen);
-	  g_object_get (settings, "gtk-icon-theme-name", &theme, NULL);
+	  g_object_get (settings, 
+			"gtk-icon-theme-name", &theme, 
+			"gtk-fallback-icon-theme", &fallback_theme, NULL);
 	}
 
       if (!theme)
@@ -424,10 +429,26 @@ update_current_theme (GtkIconTheme *icon_theme)
 	  g_free (priv->current_theme);
 	  priv->current_theme = theme;
 
-	  do_theme_change (icon_theme);
+	  changed = TRUE;
 	}
       else
 	g_free (theme);
+
+      if ((priv->fallback_theme && !fallback_theme) ||
+	  (!priv->fallback_theme && fallback_theme) ||
+	  (priv->fallback_theme && fallback_theme &&
+	   strcmp (priv->fallback_theme, fallback_theme) != 0))
+	{
+	  g_free (priv->fallback_theme);
+	  priv->fallback_theme = fallback_theme;
+
+	  changed = TRUE;
+	}
+      else
+	g_free (fallback_theme);
+
+      if (changed)
+	do_theme_change (icon_theme);
     }
 }
 
@@ -501,6 +522,8 @@ gtk_icon_theme_set_screen (GtkIconTheme *icon_theme,
 			G_CALLBACK (display_closed), icon_theme);
       g_signal_connect (settings, "notify::gtk-icon-theme-name",
 			G_CALLBACK (theme_changed), icon_theme);
+      g_signal_connect (settings, "notify::gtk-fallback-icon-theme-name",
+			G_CALLBACK (theme_changed), icon_theme);
     }
 
   update_current_theme (icon_theme);
@@ -1025,8 +1048,10 @@ load_themes (GtkIconTheme *icon_theme)
   priv->all_icons = g_hash_table_new (g_str_hash, g_str_equal);
   
   insert_theme (icon_theme, priv->current_theme);
-  
-  /* Always look in the "default" icon theme */
+
+  /* Always look in the "default" icon theme, and in a fallback theme */
+  if (priv->fallback_theme)
+    insert_theme (icon_theme, priv->fallback_theme);
   insert_theme (icon_theme, DEFAULT_THEME_NAME);
   priv->themes = g_list_reverse (priv->themes);
 
@@ -1130,6 +1155,7 @@ ensure_valid_themes (GtkIconTheme *icon_theme)
 {
   GtkIconThemePrivate *priv = icon_theme->priv;
   GTimeVal tv;
+  gboolean was_valid = priv->themes_valid;
 
   if (priv->themes_valid)
     {
@@ -1143,7 +1169,7 @@ ensure_valid_themes (GtkIconTheme *icon_theme)
     {
       load_themes (icon_theme);
       
-      if (!priv->check_reload)
+      if (!priv->check_reload && was_valid && priv->screen)
 	{	  
 	  static GdkAtom atom_iconthemes = GDK_NONE;
 	  GdkEvent *event = gdk_event_new (GDK_CLIENT_EVENT);
@@ -1484,6 +1510,21 @@ gtk_icon_theme_get_icon_sizes (GtkIconTheme *icon_theme,
 	}
     }
 
+  if (icon_theme_builtin_icons)
+    {
+      GList *icons;
+
+      icons = g_hash_table_lookup (icon_theme_builtin_icons, icon_name);
+      
+      while (icons)
+        {
+	  BuiltinIcon *icon = icons->data;
+	
+	  g_hash_table_insert (sizes, GINT_TO_POINTER (icon->size), NULL);
+          icons = icons->next;
+        }      
+    }
+
   r = result = g_new0 (gint, g_hash_table_size (sizes) + 1);
 
   g_hash_table_foreach (sizes, add_size, &r);
@@ -1893,7 +1934,7 @@ theme_lookup_icon (IconTheme          *theme,
   if (min_dir)
     {
       GtkIconInfo *icon_info = icon_info_new ();
-      gboolean has_icon_file;
+      gboolean has_icon_file = FALSE;
       
       suffix = theme_dir_get_icon_suffix (min_dir, icon_name, &has_icon_file);
       suffix = best_suffix (suffix, allow_svg);
